@@ -7,10 +7,10 @@ __all__ = [
     "input_converter",
     "output_converter",
     "from_data",
-    "from_stringified",
+    "items_to_data",
     "to_data",
     "properties",
-    "fields_set",
+    "get_fields_set",
     "get_fields",
     "mark_set_fields",
     "unmark_set_fields",
@@ -23,18 +23,17 @@ __all__ = [
     "set_type_hints",
     "ValidationError",
     "Discard",
-    "ValidatorResult",
+    "add_validator",
     "validate",
     "validator"
 ]
 
 import apischema.std_types  # Populate custom handlers for stdlib
 from .alias import alias
-from .conversion import (converter, field_input_converter,
-                         field_output_converter, inout_model, input_converter,
-                         output_converter)
-from .data import from_data, from_stringified, to_data
-from .fields import (fields_set, get_fields, mark_set_fields, unmark_set_fields,
+from .conversion import (converter, field_input_converter, field_output_converter,
+                         inout_model, input_converter, output_converter)
+from .data import from_data, items_to_data, to_data
+from .fields import (get_fields, get_fields_set, mark_set_fields, unmark_set_fields,
                      with_fields_set)
 from .ignore import Ignored
 from .json_schema import build_input_schema, build_output_schema
@@ -43,6 +42,6 @@ from .properties import properties
 from .schema import schema
 from .typing import set_type_hints
 from .validation.errors import ValidationError
-from .validation.validator import Discard, ValidatorResult, validate, validator
+from .validation.validator import Discard, add_validator, validate, validator
 
 del apischema
